# *****************************************************************
#
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2018, 2019. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# *****************************************************************

import time
import sys
import os
import numpy as np
import argparse
from sklearn.metrics import roc_auc_score

sys.path.insert(0, os.path.abspath('../'))

CLI=argparse.ArgumentParser()

CLI.add_argument(
    "--skip_xgboost",
    action = 'store_true',
    default=False
)
CLI.add_argument(
    "--data_path",
    type=str,
    default="."
)
CLI.add_argument(
    "--num_threads",
    type=int,
    default=16
)
CLI.add_argument(
    "--use_gpu",
    action = 'store_true',
    default=False
)

args = CLI.parse_args()

# Load the input datasets
t0 = time.time()
X_train = np.load(args.data_path + "/data/epsilon.X_train.npy")
X_test  = np.load(args.data_path + "/data/epsilon.X_test.npy")
y_train = np.load(args.data_path + "/data/epsilon.y_train.npy")
y_test  = np.load(args.data_path + "/data/epsilon.y_test.npy")
print("Data load time (s):  {0:.2f}".format(time.time()-t0))

# Convert the -1,1 labels to 0,1 required by xgboost (v0.9)
y_test  = (y_test + 1)/2
y_train = (y_train + 1)/2

# Fix the random state
np.random.seed(43)

# Import BoostingMachine from snap_ml or pai4sk
from pai4sk import BoostingMachine
# from snap_ml import BoostingMachine

max_depth = 6
num_round = 100

# Define training parameters
params = {}
params['num_round'] = num_round
params['learning_rate'] = 0.1
params['subsample'] = 1.0
params['colsample_bytree'] = 1.0

# In constrast to other boosting frameworks, pai4sk's BoostingMachine model dose
# not utilize a fixed maximal tree depth at each boosting iteration.
# Instead, the tree depth is sampled at each boosting iteration according to a
# discrete uniform distribution from [min_max_depth, max_max_depth]. 
params['min_max_depth'] = max_depth
# For a fair runtime evaluation against xgboost we set max_max_depth to be equal to min_max_depth
params['max_max_depth'] = params['min_max_depth']

# Number of CPU threads
params['nthread'] = args.num_threads
params['seed'] = 42
params['hist_nbins'] = 256

# Create a pai4sk BoostingMachine object
booster = BoostingMachine(objective='logloss',
          num_round=params['num_round'],
          min_max_depth=params['min_max_depth'],
          max_max_depth=params['max_max_depth'],
          verbose=False,
          learning_rate=params['learning_rate'],
          random_state=params['seed'],
          n_threads=params['nthread'],
          subsample=params['subsample'],
          colsample_bytree=params['colsample_bytree'],
          hist_nbins=params['hist_nbins'])
          
if args.use_gpu:
    booster = BoostingMachine(objective='logloss',
          num_round=params['num_round'],
          min_max_depth=params['min_max_depth'],
          max_max_depth=params['max_max_depth'],
          verbose=False,
          learning_rate=params['learning_rate'],
          random_state=params['seed'],
          n_threads=params['nthread'],
          subsample=params['subsample'],
          colsample_bytree=params['colsample_bytree'],
          use_gpu=True,
          gpu_id=0,
          hist_nbins=params['hist_nbins'])

os.environ['OMP_NUM_THREADS'] = str(params['nthread'])

# Train a pai4sk BoostingMachine model
t0 = time.time()
booster.fit(X_train, y_train)
t_fit = time.time()-t0

# Predict with the BoostingMachine model
yhat_test = booster.predict(X_test)

if args.use_gpu == False:
    print("[pai4sk - BoostingMachine-CPU] training_time %f roc_auc_score %f " % (t_fit,roc_auc_score(y_test, yhat_test)))
else:
    print("[pai4sk - BoostingMachine-GPU] training_time %f roc_auc_score %f " % (t_fit,roc_auc_score(y_test, yhat_test)))


if args.skip_xgboost:
    print('Execution using xgboost is skipped.')
    exit()

# Import xgbooster from XGBoost
import xgboost as xgb

# Prepare the train/test input datasets
dtrain = xgb.DMatrix(X_train, y_train)
dtest = xgb.DMatrix(X_test, y_test)

# Define training parameters
params['objective'] = 'binary:logistic'
params['num_round'] = num_round
params['learning_rate'] = 0.1
params['subsample'] = 1.0
params['colsample_bytree'] = 1.0
params['max_depth'] = max_depth
params['lambda'] = 0.0
params['min_child_weight'] = 0.0

if args.use_gpu == False:
    params['tree_method'] = 'hist'
else:
    params['tree_method'] = 'gpu_hist'
    params['n_gpus'] = 1
    params['gpu_id'] = 0

params['max_bin'] = 256
params['nthread'] = args.num_threads
params['seed'] = 42

if args.use_gpu == False:
    # Training using the XGBoost CPU solver
    os.environ['CUDA_VISIBLE_DEVICES'] = '-1'

# Train an XGBoost model
t0 = time.time()
gbm = xgb.train(params, dtrain, params['num_round'])
t_fit = time.time() - t0

# Predict with the XGBoost model
y_test  = dtest.get_label()
yhat_test = gbm.predict(dtest, output_margin=False)

if args.use_gpu == False:
    print("[XGBoost-CPU] training_time %f roc_auc_score %f " % (t_fit,roc_auc_score(y_test, yhat_test)))
else:
    print("[XGBoost-GPU] training_time %f roc_auc_score %f " % (t_fit,roc_auc_score(y_test, yhat_test)))

